; WALK.asm
; --------------------------------------------------------
;   Example for XESS board 8031, linked with VTOP FPGA
; --------------------------------------------------------
; Bertrand Cuzeau
; alse@compuserve.com
;
; This works as follows:
;  - a bit is walked four times on the LED display
;  - a seed is then written in the LFSR
;  - the LFSR is shifted 32 times, read back,
;    and written to the LED
;  - the system cycles back to the beginning of the test
;
; This is probably not a very elegant code, but I am
; not familiar with the 8031...
;
; Don't forget to Pulse D0 in the parallel port (reset) !

$MOD51	; Model type

STACKTOP EQU    70H           ; start of stack (grows up)
LEDREG   EQU    0F000H        ; LED register address
SCAN     EQU    0E000H        ; Scancode, type, and done address

DSEG	; Data segment
        ORG     30H
CNTR:   DS      1             ; counter for wait routine
CNT2:   DS      1             ; counter for bit walk test

CSEG	; Code segment

        ORG     0000H         ; program starts at 0 after reset

START:
        MOV     SP,#STACKTOP  ; initialize stack pointer ...

; --- Walking bit LED Test ---
START2:

	MOV R2,#0	; initalize our operands
	MOV R1,#0

	MOV DPTR,#SCAN	; uC Now has Dptr pointing to out Scancode Register
	
WAIT_FOR_DONE_DROP1:

	MOVX A,@DPTR	; uC Is requesting information from the FPGA
		
	ANL A,#32	; This will single out the done bit

	CJNE A,#32,WAIT_FOR_VAILD_INPUT1
			; This will look at the done bit and see if has gone low

	JMP WAIT_FOR_DONE_DROP1
			; This will redo the past steps till done drops low	

WAIT_FOR_VAILD_INPUT1:

	MOVX A,@DPTR	; uC Is requesting information from the FPGA
	MOV R3,A	; Saves the input value for later anaylzing
	
	ANL A,#32	; This will single out the done bit

	CJNE A,#0,HAVE_VALID_INPUT1
			; This will look at the done bit and see if has gone high

	JMP WAIT_FOR_VAILD_INPUT1
			; This will redo the past steps till done raises high	

HAVE_VALID_INPUT1:

	MOV A,R3	; Reloads the saved input value
	ANL A,#16	; This will single out the type bit

	CJNE A,#0,HAVE_OPERATION1

HAVE_NUMBER1:

	MOV A,R3		; Saves the number inputted into a safe memory location
	MOV R4,A
	
	MOV A,R3
	MOV DPTR,#LEDREG	; This tells the uC to now target the LED register
	MOVX @DPTR,A		; This displays the current operand

	MOV DPTR,#SCAN		; This tells the uC to now target the Scancode Register

	JMP WAIT_FOR_DONE_DROP1
				; Goes back to looking of a new operand or operator

HAVE_OPERATION1:

	MOV A,#0
	MOV DPTR,#LEDREG
	MOVX @DPTR,A

	MOV A,R3		; Saves the operation code for later
	MOV R5,A

	MOV DPTR,#SCAN
	
WAIT_FOR_DONE_DROP2:

	MOVX A,@DPTR	; uC Is requesting information from the FPGA
		
	ANL A,#32	; This will single out the done bit

	CJNE A,#32,WAIT_FOR_VAILD_INPUT2
			; This will look at the done bit and see if has gone low

	JMP WAIT_FOR_DONE_DROP2
			; This will redo the past steps till done drops low	

WAIT_FOR_VAILD_INPUT2:

	MOVX A,@DPTR	; uC Is requesting information from the FPGA
	MOV R3,A	; Saves the input value for later anaylzing
	
	ANL A,#32	; This will single out the done bit

	CJNE A,#0,HAVE_VALID_INPUT2
			; This will look at the done bit and see if has gone high

	JMP WAIT_FOR_VAILD_INPUT2
			; This will redo the past steps till done raises high

HAVE_VALID_INPUT2:

	MOV A,R3	; Reloads the saved input value
	ANL A,#16	; This will single out the type bit

	CJNE A,#0,HAVE_OPERATION2

HAVE_NUMBER2:

	MOV A,R3		; Saves the number inputted into a safe memory location
	MOV R6,A

	MOV A,R3
	MOV DPTR,#LEDREG	; This tells the uC to now target the LED register
	MOVX @DPTR,A		; This displays the current operand

	MOV DPTR,#SCAN		; This tells the uC to now target the Scancode Register

	JMP WAIT_FOR_DONE_DROP2
				; Goes back to looking of a new operand or operator

HAVE_OPERATION2:

	MOV A,R5		; Reload operation into A
	ANL A,#15		; Singles out the operation code

	CJNE A,#6,NAS		; The uC finds if the operation was a multiplcation
	
MULTIPLCATION:			; This multiplication works as if it is raising the number 
				; to a power

	PUSH ACC		; Stores accumulator to stack
	MOV A,R6		; Moves first number from R6 to accumulator
	MOV R2,A		; Stores first number into R2
	MOV A,R4		; Moves second number from R4 to accumulator
	MOV R1,A		; Stores second number into R1
	POP ACC			; Pops accumulator value from stack
	MOV A,#0		; Moves the value zero into accumulator
	MOV R3,A		; Stores R3 with the value of zero

MULT1:

	PUSH ACC		; Push current value of accumulator to stack
	MOV A,R3		; Loads the accumulator with a zero number value
	MOV B,R2		; Loads B with the first number
	CJNE A,B,MULT2		; Compares A to B if they are not equal it jumps to Mult2
	POP ACC			; Pops accumulator value from stack, the end result of mutl2

	JMP MULTDONE		; Jumps to Multdone

MULT2:

	POP ACC			; Pops accumulator value from stack
	ADD A,R1		; Adds first number stored in R1 to the accumulator value
	MOV R4,A		; Stores accumulator to R4 
	MOV A,R3		; Moves value of R3 into accumulator
	ADD A,#1		; Adds one to the accumulator
	MOV R3,A		; Stores value into R3 which is used to compare with R2 
	MOV A,R4		; Stores R4 to accumulator
	JMP MULT1		; Jumps back to Mult1

MULTDONE:

	JMP END_

NAS:
	CJNE A,#0Fh,ADDITION

SUBTRACTION:

	MOV A,R4
	SUBB A,R6

	JMP END_

ADDITION:

	MOV A,R4
	ADD A,R6
	
END_:

	MOV DPTR,#LEDREG	; Gives uC the address of the LEDREG
	MOVX @DPTR,A		; uC sends information to the LEDREG

	MOV DPTR,#SCAN		; Tells the uC to target the Scancode register

	JMP WAIT_FOR_DONE_DROP1	; This restarts the calculator

; --- STOP ---
        JMP     START2
        JMP     $             ; Suicide... (wait for a reset)

	END


